####################################################################################
####################################################################################
####                                                                            ####
####                              R code - MAGBLUP                              ####
####                              Flint-Dent Panel                              ####
####                                 S.Rio 2020                                 ####
####                                                                            ####
####  Associated publication (Rio et al. 2020):                                 ####
####    Accounting for group-specific allele effects and admixture in genomic   ####
####    predictions: theory and experimental evaluation in maize                ####
####                                                                            ####
####  Structure of the script:                                                  ####
####    1 - Inital settings and data loading (genotypes and phenotypes)         ####
####    2 - Simulation of traits with group-specific allele effects             ####
####    3 - Variance estimates of GBLUP-RI, GBLUP-RAE, MAGBLUP-RI and           ####
####        MAGBLUP-RAE                                                         ####
####    4 - Genomic prediction accuracy based on SHO Cross Validations          #### 
####                                                                            ####
####################################################################################
####################################################################################


#########
####  1 - Inital settings and data loading (genotypes and phenotypes)
#########

## Remove all objects from the R environment
rm(list=ls())

## Set working directory (to be adapted)
setwd("/home")

## Install unsatisfied dependencies
required <- c("MM4LMM")
not_installed <- setdiff(required, rownames(installed.packages()))
if (length(not_installed) > 0) install.packages(not_installed)

## R packages to be used:
## MM4LMM : parameters inference
library(MM4LMM)

## Fonctions to be used:
source("./MAGBLUP_RFunctions_FlintDent.R")

## Load data including:
##  - geno : genotyping matrix (970 x 482078) with bi-allelic genotypic scores of each homozygous inbred line (coded 0/1)
##  - dent_ancestry : matrix (970 x 482078) indicating the ancestry of each allele (0: flint genetic group, 1: dent genetic group)
##  - pheno : dataframe (970 x 5) with phenotypic values of each line for male (MF) and female (FF) flowering time, 
##    plant height (PH), ear leaf number (ELN) and total number of leaves (TNL) 
##  - SNP_info : dataframe (482078 x 2) with information on each SNP (chromosome and position (B73 RefGen_v2))
##  - genetic background : data.frame (970 x 1) indicating the genetic background of the line (Dent, Flint or Admixed)
geno <- as.matrix(read.csv("./Genotypes_FlintDent_970L_482078M.csv",row.names = 1))
dent_ancestry <- as.matrix(read.csv("./Dent_SNP_ancestry_FlintDent_970L_482078M.csv",row.names = 1))
pheno_real <- read.csv("./Phenotypes_FlintDent_970L_5T.csv",row.names = 1)
SNP_info <- read.csv("./SNP_info_FlintDent_482078M.csv",row.names = 1)
genetic_background <- read.csv("./Genetic_background_FlintDent_970L.csv",row.names = 1)

data_FlinfDent <- list(geno = geno, dent_ancestry = dent_ancestry, 
                       pheno = pheno_real,SNP_info = SNP_info, 
                       genetic_background = genetic_background)

## OPTIONAL: SNP subset to be sampled 
snp_subset <- 5000
sample_snp <- sort(sample(1:ncol(geno),snp_subset))
geno <- data_FlinfDent$geno[,sample_snp]
dent_ancestry <- data_FlinfDent$dent_ancestry[,sample_snp]
SNP_info <- data_FlinfDent$SNP_info[sample_snp,]


#########
####  2 - Simulation of traits with group-specific allele effects
#########

## Number of QTLs
nb_qtl <- 1000

## Variances of allele effects for each genetic configuration
## sigma2_gamma, sigma2_delta_D and sigma2_delta_F
sigma2_effects <- list(c(2,0,0),c(2,1,3),c(0,1,3))
names(sigma2_effects) <- c("Main","Main+Dev.","Dev.")

## Group-specific reference allele frequency
f_D <- colMeans(geno[which(genetic_background$genetic_background=="Dent"),]) ## Dent 
f_F <- colMeans(geno[which(genetic_background$genetic_background=="Flint"),]) ## Flint 

## Simulation of QTL effects and calculation of reference variances
data_simulated_effects <- generate_effects(sigma2_effects,f_D,f_F,nb_qtl)
simulated_effects <- data_simulated_effects$effects ## Simulated effects
QTL_sample_sim <- data_simulated_effects$QTL_sample ## SNPs declared as QTLs

## Reference variances
Var_ref_MAGBLUP_RI_sim <- data_simulated_effects$var_MAGBLUP_RI ## MAGBLUP-RI
t(sapply(Var_ref_MAGBLUP_RI_sim,c))
Var_ref_MAGBLUP_RAE_sim <- data_simulated_effects$var_MAGBLUP_RAE ## MAGBLUP-RAE
t(sapply(Var_ref_MAGBLUP_RAE_sim,c))

## Heritability of simulated traits
h2 <- 0.8 

## Simulation of phenotypes
data_generate_pheno <- generate_pheno(geno,dent_ancestry,simulated_effects,QTL_sample_sim,h2)
TBV_sim <- data_generate_pheno$TBV ## simulated true breeding value 
pheno_sim <- data_generate_pheno$pheno ## Simulated phenotypes


#########
#### 3 - Variance estimates of GBLUP-RI, GBLUP-RAE, MAGBLUP-RI and MAGBLUP-RAE
#########

## Reference phenotypes (real or simulated) TO BE CHOSEN
# pheno <- pheno_real
pheno <- pheno_sim

## GBLUP-RI inference
List_Var_GBLUP_RI <- GBLUP_RI_Var(geno = geno)
model_GBLUP_RI <- lapply(1:ncol(pheno),function(i)
  MMEst(Y = pheno[,i],VarList  = List_Var_GBLUP_RI))
var_GBLUP_RI <- sapply(model_GBLUP_RI,function(i)i$NullModel$Sigma2)
rownames(var_GBLUP_RI) <- c("Sigma2_G","Sigma2_E")
colnames(var_GBLUP_RI) <- colnames(pheno)
round(var_GBLUP_RI,2) ## Variance estimates

## GBLUP-RAE inference
List_Var_GBLUP_RAE <- GBLUP_RAE_Var(geno = geno)
model_GBLUP_RAE <- lapply(1:ncol(pheno),function(i)
  MMEst(Y = pheno[,i],VarList  = List_Var_GBLUP_RAE))
var_GBLUP_RAE  <- sapply(model_GBLUP_RAE,function(i)i$NullModel$Sigma2)
rownames(var_GBLUP_RAE) <- c("Sigma2_U","Sigma2_E")
colnames(var_GBLUP_RAE) <- colnames(pheno)
round(var_GBLUP_RAE,2) ## Variance estimates

## MAGBLUP-RI inference
admixture <- cbind(rowMeans(dent_ancestry),1-rowMeans(dent_ancestry))
List_Var_MAGBLUP_RI <- MAGBLUP_RI_Var(geno,dent_ancestry)
model_MAGBLUP_RI <- lapply(1:ncol(pheno),function(i)
  MMEst(Y = pheno[,i],Cofactor = admixture,VarList  = List_Var_MAGBLUP_RI))
var_MAGBLUP_RI <- sapply(model_MAGBLUP_RI,function(i)i[[1]]$Sigma2)
rownames(var_MAGBLUP_RI) <- c("Sigma2_S","Sigma2_GD","Sigma2_GF","Sigma2_E")
colnames(var_MAGBLUP_RI) <- colnames(pheno)
round(var_MAGBLUP_RI,2) ## Variance estimates

## MAGBLUP-RAE inference 
List_Var_MAGBLUP_RAE <- MAGBLUP_RAE_Var(geno,dent_ancestry)
model_MAGBLUP_RAE <- lapply(1:ncol(pheno),function(i)
  MMEst(Y = pheno[,i],VarList  = List_Var_MAGBLUP_RAE))
var_MAGBLUP_RAE <- sapply(model_MAGBLUP_RAE,function(i)i[[1]]$Sigma2)
rownames(var_MAGBLUP_RAE) <- c("Sigma2_U","Sigma2_UD","Sigma2_UF","Sigma2_E")
colnames(var_MAGBLUP_RAE) <- colnames(pheno)
round(var_MAGBLUP_RAE,2) ## Variance estimates


#########
####  4 - Genomic prediction accuracy based on SHO Cross Validations
#########

## Reference phenotypes (real or simulated) TO BE CHOSEN
# pheno <- pheno_real
pheno <- pheno_sim

## Illustration with scenario DFA_DFA (dent, flint and admixed in both training and predicted set)
nb_ind <- nrow(geno)
nb_TS_ind <- 210
nb_PS_ind <- 90
sample_DFA_DFA <- SHO_sampling(nb_TS = rep(nb_TS_ind/3,3),nb_PS = rep(nb_PS_ind/3,3),
                              genetic_background = genetic_background)

## Phenotypes of TS individuals
Y_TS_DFA_DFA <- pheno[sample_DFA_DFA$TS,]

## Incidence matrices for random components 
Z_list_TS_DFA_DFA <- lapply(1:4,function(i)diag(nb_ind)[sample_DFA_DFA$TS,])

## GBLUP-RI inference and prediction
model_GBLUP_RI_DFA_DFA <- lapply(1:ncol(Y_TS_DFA_DFA),function(i)
  MMEst(Y = Y_TS_DFA_DFA[,i],VarList  = List_Var_GBLUP_RI,ZList = Z_list_TS_DFA_DFA[1:2]))
GEBV_GBLUP_RI <- sapply(1:ncol(Y_TS_DFA_DFA),function(i){
  fixed_part <- model_GBLUP_RI_DFA_DFA[[i]]$NullModel$Beta
  random_part <- Reduce("+",BLUP_MM4LMM(model = model_GBLUP_RI_DFA_DFA[[i]],
                                     Y = Y_TS_DFA_DFA[,i],
                                     X = matrix(1,nrow = nb_TS_ind,ncol = 1),
                                     List_var = List_Var_GBLUP_RI,
                                     List_Z = Z_list_TS_DFA_DFA[1:2],
                                     Component = 1))
  res <- fixed_part+random_part
  return(res)
})

## GBLUP-RAE inference and prediction
model_GBLUP_RAE_DFA_DFA <- lapply(1:ncol(Y_TS_DFA_DFA),function(i)
  MMEst(Y = Y_TS_DFA_DFA[,i],VarList  = List_Var_GBLUP_RAE,ZList = Z_list_TS_DFA_DFA[1:2]))
GEBV_GBLUP_RAE <- sapply(1:ncol(Y_TS_DFA_DFA),function(i){
  fixed_part <- model_GBLUP_RAE_DFA_DFA[[i]]$NullModel$Beta
  random_part <- Reduce("+",BLUP_MM4LMM(model = model_GBLUP_RI_DFA_DFA[[i]],
                                     Y = Y_TS_DFA_DFA[,i],
                                     X = matrix(1,nrow = nb_TS_ind,ncol = 1),
                                     List_var = List_Var_GBLUP_RAE,
                                     List_Z = Z_list_TS_DFA_DFA[1:2],
                                     Component = 1))
  res <- fixed_part+random_part
  return(res)
})

## MAGBLUP-RI inference and prediction
admixture_TS_DFA_DFA <- matrix(admixture[sample_DFA_DFA$TS,2])
model_MAGBLUP_RI_DFA_DFA <- lapply(1:ncol(Y_TS_DFA_DFA),function(i)
  MMEst(Y = Y_TS_DFA_DFA[,i],Cofactor = admixture_TS_DFA_DFA,
        VarList  = List_Var_MAGBLUP_RI,ZList = Z_list_TS_DFA_DFA))
GEBV_MAGBLUP_RI <- sapply(1:ncol(Y_TS_DFA_DFA),function(i){
  fixed_part <- admixture%*%model_MAGBLUP_RI_DFA_DFA[[i]]$NullModel$Beta
  random_part <- Reduce("+",BLUP_MM4LMM(model = model_MAGBLUP_RI_DFA_DFA[[i]],
                                     Y = Y_TS_DFA_DFA[,i],
                                     X = cbind(1,admixture_TS_DFA_DFA),
                                     List_var = List_Var_MAGBLUP_RI,
                                     List_Z = Z_list_TS_DFA_DFA,
                                     Component = c(1:3)))
  res <- fixed_part+random_part
  return(res)
})

## MAGBLUP-RAE inference and prediction
model_MAGBLUP_RAE_DFA_DFA <- lapply(1:ncol(Y_TS_DFA_DFA),function(i)
  MMEst(Y = Y_TS_DFA_DFA[,i],VarList  = List_Var_MAGBLUP_RAE,ZList = Z_list_TS_DFA_DFA))
GEBV_MAGBLUP_RAE=sapply(1:ncol(Y_TS_DFA_DFA),function(i){
  fixed_part <- model_MAGBLUP_RAE_DFA_DFA[[i]]$NullModel$Beta
  random_part <- Reduce("+",BLUP_MM4LMM(model = model_MAGBLUP_RAE_DFA_DFA[[i]],
                                     Y = Y_TS_DFA_DFA[,i],
                                     X = matrix(1,nrow = nb_TS_ind,ncol = 1),
                                     List_var = List_Var_MAGBLUP_RAE,
                                     List_Z = Z_list_TS_DFA_DFA,
                                     Component = c(1:3)))
  res <- fixed_part+random_part
  return(res)
})

## Predictive ability using phenotypes (for PS) according to GS model and trait
pred_ability_DFA_DFA <- sapply(1:ncol(Y_TS_DFA_DFA),function(i)
  cor(cbind(GEBV_GBLUP_RI[,i],GEBV_GBLUP_RAE[,i],GEBV_MAGBLUP_RI[,i],
            GEBV_MAGBLUP_RAE[,i],pheno[,i])[sample_DFA_DFA$PS,])[1:4,5])
rownames(pred_ability_DFA_DFA) <- c("GBLUP-RI","GBLUP-RAE","MAGBLUP-RI","MAGBLUP-RAE")
colnames(pred_ability_DFA_DFA) <- colnames(pheno)
round(pred_ability_DFA_DFA,2) 

# Accuracy using TBV (for PS) according to GS model and trait (ONLY FOR SIMULATED TRAITS)
accuracy_DFA_DFA <- sapply(1:ncol(Y_TS_DFA_DFA),function(i)
  cor(cbind(GEBV_GBLUP_RI[,i],GEBV_GBLUP_RAE[,i],GEBV_MAGBLUP_RI[,i],
            GEBV_MAGBLUP_RAE[,i],TBV_sim[,i])[sample_DFA_DFA$PS,])[1:4,5])
rownames(accuracy_DFA_DFA) <- c("GBLUP-RI","GBLUP-RAE","MAGBLUP-RI","MAGBLUP-RAE")
colnames(accuracy_DFA_DFA) <- colnames(TBV_sim)
round(accuracy_DFA_DFA,2) 

